@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 4415 2009-01-09 16:17:15Z wwalker $"
-__version__   = "$Revision: 4415 $"
-__date__      = "$Date: 2009-01-09 11:17:15 -0500 (Fri, 09 Jan 2009) $"
+__id__        = "$Id: braillegenerator.py 4446 2009-01-21 19:54:21Z wwalker $"
+__version__   = "$Revision: 4446 $"
+__date__      = "$Date: 2009-01-21 14:54:21 -0500 (Wed, 21 Jan 2009) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1386,7 +1386,8 @@ class BrailleGenerator:
             image = None
         if (not displayedText or len(displayedText) == 0) and image:
             if image.imageDescription:
-                regions[0].append(image.imageDescription)
+                regions[0].append(braille.Component(obj,
+                                  image.imageDescription))
             [cellRegions, focusRegion] = self._getBrailleRegionsForImage(obj)
             regions[0].extend(cellRegions)
 
