@@ -33,9 +33,9 @@ created in their own module.  The module defining the Script subclass
 is also required to have a 'getScript(app)' method that returns an
 instance of the Script subclass.  See default.py for an example."""
 
-__id__        = "$Id: script.py 3957 2008-06-04 16:43:04Z richb $"
-__version__   = "$Revision: 3957 $"
-__date__      = "$Date: 2008-06-04 12:43:04 -0400 (Wed, 04 Jun 2008) $"
+__id__        = "$Id: script.py 4103 2008-08-15 11:07:31Z wwalker $"
+__version__   = "$Revision: 4103 $"
+__date__      = "$Date: 2008-08-15 07:07:31 -0400 (Fri, 15 Aug 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -46,8 +46,10 @@ import keybindings
 import orca_state
 import settings
 import speechgenerator
+import structural_navigation
 import where_am_I
 import bookmarks
+import tutorialgenerator
 
 class Script:
     """The specific focus tracking scripts for applications.
@@ -76,6 +78,7 @@ class Script:
         #
         self.presentIfInactive = True
 
+        self.structuralNavigation = self.getStructuralNavigation()
         self.inputEventHandlers = {}
         self.pointOfReference = {}
         self.setupInputEventHandlers()
@@ -88,6 +91,7 @@ class Script:
         self.whereAmI = self.getWhereAmI()
         self.bookmarks = self.getBookmarks()
         self.voices = settings.voices
+        self.tutorialGenerator = self.getTutorialGenerator()
 
         self.flatReviewContextClass = flat_review.Context
 
@@ -184,6 +188,32 @@ class Script:
         """
         return speechgenerator.SpeechGenerator(self)
 
+    def getTutorialGenerator(self):
+        """Returns the tutorial generator for this script.
+        """
+        return tutorialgenerator.TutorialGenerator(self)
+
+    def getEnabledStructuralNavigationTypes(self):
+        """Returns a list of the structural navigation object types
+        enabled in this script.
+        """
+        return []
+
+    def getStructuralNavigation(self):
+        """Returns the 'structural navigation' class for this script.
+        """
+        types = self.getEnabledStructuralNavigationTypes()
+        return structural_navigation.StructuralNavigation(self, types)
+
+    def useStructuralNavigationModel(self):
+        """Returns True if we should use structural navigation. Most
+        scripts will have no need to override this.  Gecko does however
+        because within an HTML document there are times when we do want
+        to use it and times when we don't even though it is enabled,
+        e.g. in a form field.
+        """
+        return self.structuralNavigation.enabled
+
     def getWhereAmI(self):
         """Returns the "where am I" class for this script.
         """
@@ -336,9 +366,19 @@ class Script:
 
         consumes = False
         if user_bindings:
-            consumes = user_bindings.getInputHandler(keyboardEvent) != None
+            handler = user_bindings.getInputHandler(keyboardEvent)
+            if handler \
+                 and handler.function in self.structuralNavigation.functions:
+                return self.useStructuralNavigationModel()
+            else:
+                consumes = handler != None
         if not consumes:
-            consumes = self.keyBindings.getInputHandler(keyboardEvent) != None
+            handler = self.keyBindings.getInputHandler(keyboardEvent)
+            if handler \
+                 and handler.function in self.structuralNavigation.functions:
+                return self.useStructuralNavigationModel()
+            else:
+                consumes = handler != None
         return consumes
 
     def processKeyboardEvent(self, keyboardEvent):
