@@ -19,9 +19,9 @@
 
 """Custom script for Evolution."""
 
-__id__        = "$Id: script.py 4337 2008-11-17 22:01:23Z joanied $"
-__version__   = "$Revision: 4337 $"
-__date__      = "$Date: 2008-11-17 17:01:23 -0500 (Mon, 17 Nov 2008) $"
+__id__        = "$Id: script.py 4608 2009-02-27 09:31:37Z joanied $"
+__version__   = "$Revision: 4608 $"
+__date__      = "$Date: 2009-02-27 04:31:37 -0500 (Fri, 27 Feb 2009) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -69,6 +69,10 @@ class Script(default.Script):
 
         self.message_panel = None
 
+        # A handle to the Spellcheck dialog.
+        #
+        self.spellCheckDialog = None
+
         # Dictionary of Setup Assistant panels already handled.
         #
         self.setupPanels = {}
@@ -84,7 +88,6 @@ class Script(default.Script):
 
         # The last locusOfFocusChanged roles hierarchy.
         #
-        self.lastRolesList = []
         self.rolesList = []
 
         # By default, don't present if Evolution is not the active application.
@@ -514,6 +517,102 @@ class Script(default.Script):
                     if text.getNSelections():
                         text.removeSelection(0)
 
+    def isSpellingSuggestionsList(self, obj):
+        """Returns True if obj is the list of spelling suggestions
+        in the spellcheck dialog.
+
+        Arguments:
+        - obj: the Accessible object of interest.
+        """
+
+        # The list of spelling suggestions is a table whose parent is
+        # a scroll pane. This in and of itself is not sufficiently
+        # unique. What makes the spell check dialog unique is the
+        # quantity of push buttons found. If we find this combination,
+        # we'll assume its the spelling dialog.
+        #
+        if obj and obj.getRole() == pyatspi.ROLE_TABLE_CELL:
+            obj = obj.parent
+
+        if not obj \
+           or obj.getRole() != pyatspi.ROLE_TABLE \
+           or obj.parent.getRole() != pyatspi.ROLE_SCROLL_PANE:
+            return False
+
+        topLevel = self.getTopLevel(obj.parent)
+        if not self.isSameObject(topLevel, self.spellCheckDialog):
+            # The group of buttons is found in a filler which is a
+            # sibling of the scroll pane.
+            #
+            for sibling in obj.parent.parent:
+                if sibling.getRole() == pyatspi.ROLE_FILLER:
+                    buttonCount = 0
+                    for child in sibling:
+                        if child.getRole() == pyatspi.ROLE_PUSH_BUTTON:
+                            buttonCount += 1
+                    if buttonCount >= 5:
+                        self.spellCheckDialog = topLevel
+                        return True
+        else:
+            return True
+
+        return False
+
+    def getMisspelledWordAndBody(self, suggestionsList, messagePanel):
+        """Gets the misspelled word from the spelling dialog and the 
+        list of words from the message body.
+
+        Arguments:
+        - suggestionsList: the list of spelling suggestions from the
+          spellcheck dialog
+        - messagePanel: the panel containing the message being checked
+          for spelling
+
+        Returns [mispelledWord, messageBody]
+        """
+
+        misspelledWord, messageBody = "", []
+
+        # Look for the "Suggestions for "xxxxx" label in the spell
+        # checker dialog panel. Extract out the xxxxx. This will be
+        # the misspelled word.
+        #
+        text = self.getDisplayedLabel(suggestionsList) or ""
+        words = text.split()
+        for word in words:
+            if word[0] in ["'", '"']:
+                misspelledWord = word[1:len(word) - 1]
+                break
+
+        if messagePanel != None:
+            allTextObjects = self.findByRole(messagePanel, pyatspi.ROLE_TEXT)
+            for obj in allTextObjects:
+                for word in self.getText(obj, 0, -1).split():
+                    messageBody.append(word)
+
+        return [misspelledWord, messageBody]
+
+    def isMessageBodyText(self, obj):
+        """Returns True if obj is in the body of an email message.
+
+        Arguments:
+        - obj: the Accessible object of interest.
+        """
+
+        try:
+            obj.queryHypertext()
+            ancestor = obj.parent.parent
+        except:
+            return False
+        else:
+            # The accessible text objects in the header at the top
+            # of the message also have STATE_MULTI_LINE. But they
+            # are inside panels which are inside table cells; the
+            # body text is not. See bug #567428.
+            #
+            return (obj.getState().contains(pyatspi.STATE_MULTI_LINE) \
+                    and ancestor.getRole() != pyatspi.ROLE_TABLE_CELL)
+
     def presentMessageLine(self, obj, newLocusOfFocus):
         """Speak/braille the line at the current text caret offset.
         """
@@ -587,11 +686,6 @@ class Script(default.Script):
         brailleGen = self.brailleGenerator
         speechGen = self.speechGenerator
 
-        # Save the previous role hierarchy list for possible comparison
-        # in section 8).
-        #
-        self.lastRolesList = self.rolesList
-
         debug.printObjectEvent(self.debugLevel,
                                event,
                                debug.getAccessibleDetails(event.source))
@@ -616,15 +710,8 @@ class Script(default.Script):
         # "text", "panel" and "unknown". If we find that, then (hopefully)
         # it's a line in the mail message and we get the utterances to
         # speak for that Text.
-
-        self.rolesList = [pyatspi.ROLE_TEXT, \
-                          pyatspi.ROLE_PANEL, \
-                          pyatspi.ROLE_UNKNOWN, \
-                          pyatspi.ROLE_PANEL, \
-                          pyatspi.ROLE_SCROLL_PANE, \
-                          pyatspi.ROLE_FILLER, \
-                          pyatspi.ROLE_SPLIT_PANE]
-        if self.isDesiredFocusedItem(event.source, self.rolesList):
+        if self.isMessageBodyText(event.source) \
+           and not event.source.getState().contains(pyatspi.STATE_EDITABLE):
             debug.println(self.debugLevel,
                           "evolution.locusOfFocusChanged - mail view: " \
                           + "current message pane: " \
@@ -1172,15 +1259,8 @@ class Script(default.Script):
         #
         # Note that this drops through to then use the default event
         # processing in the parent class for this "focus:" event.
-
-        self.rolesList = [pyatspi.ROLE_TEXT, \
-                          pyatspi.ROLE_PANEL, \
-                          pyatspi.ROLE_UNKNOWN, \
-                          pyatspi.ROLE_PANEL, \
-                          pyatspi.ROLE_SCROLL_PANE, \
-                          pyatspi.ROLE_FILLER, \
-                          pyatspi.ROLE_PANEL]
-        if self.isDesiredFocusedItem(event.source, self.rolesList):
+        if self.isMessageBodyText(event.source) \
+           and event.source.getState().contains(pyatspi.STATE_EDITABLE):
             debug.println(self.debugLevel,
                           "evolution.locusOfFocusChanged - mail " \
                           + "compose window: message area.")
@@ -1197,9 +1277,9 @@ class Script(default.Script):
                 if self.isSameObject(event.source.parent,
                                      orca_state.locusOfFocus.parent):
                     lastKey = orca_state.lastNonModifierKeyEvent.event_string
-                    if self.lastRolesList == self.rolesList and \
-                       lastKey not in ["Left", "Right", "Up", "Down", \
-                                       "Home", "End", "Return"]:
+                    if self.isMessageBodyText(orca_state.locusOfFocus) \
+                       and lastKey not in ["Left", "Right", "Up", "Down",
+                                           "Home", "End", "Return", "Tab"]:
                         return
 
                     # If the last keyboard event was a "same line" 
@@ -1225,15 +1305,8 @@ class Script(default.Script):
         # caret currently is, and use this to speak a selection of the
         # surrounding text, to give the user context for the current misspelt
         # word.
-
-        self.rolesList = [pyatspi.ROLE_TABLE, \
-                         pyatspi.ROLE_SCROLL_PANE, \
-                         pyatspi.ROLE_PANEL, \
-                         pyatspi.ROLE_PANEL, \
-                         pyatspi.ROLE_PANEL, \
-                         pyatspi.ROLE_FILLER, \
-                         pyatspi.ROLE_DIALOG]
-        if self.isDesiredFocusedItem(event.source, self.rolesList):
+        if self.isSpellingSuggestionsList(event.source) \
+           or self.isSpellingSuggestionsList(newLocusOfFocus):
             debug.println(self.debugLevel,
                       "evolution.locusOfFocusChanged - spell checking dialog.")
 
@@ -1241,42 +1314,11 @@ class Script(default.Script):
             #
             self.updateBraille(orca_state.locusOfFocus)
 
-            # Look for the "Suggestions for 'xxxxx' label in the spell
-            # checker dialog panel. Extract out the xxxxx. This will be the
-            # misspelt word.
-            #
-            panel = event.source.parent.parent
-            allLabels = self.findByRole(panel, pyatspi.ROLE_LABEL)
-            found = False
-            for label in allLabels:
-                if not found:
-                    text = self.getDisplayedText(label)
-                    if text:
-                        tokens = text.split()
-                    else:
-                        tokens = []
-                    for token in tokens:
-                        if token.startswith("'"):
-                            badWord = token
-                            badWord = badWord[1:len(badWord)-1]
-                            found = True
-                            break
-
-            # If we have a handle to the HTML message panel, then extract out
-            # all the text objects, and create a list of all the words found
-            # in them.
-            #
-            if self.message_panel != None:
-                allTokens = []
-                panel = self.message_panel
-                allText = self.findByRole(panel, pyatspi.ROLE_TEXT)
-                for i in range(0, len(allText)):
-                    text = self.getText(allText[i], 0, -1)
-                    tokens = text.split()
-                    allTokens += tokens
-
+            if not self.pointOfReference.get('activeDescendantInfo'):
+                [badWord, allTokens] = \
+                    self.getMisspelledWordAndBody(event.source, 
+                                                  self.message_panel)
                 self.speakMisspeltWord(allTokens, badWord)
-                return
 
         # 10) Mail view: message area - attachments.
         #
@@ -1447,8 +1489,8 @@ class Script(default.Script):
         # prior to speaking the actual message that gets focus.
         # See bug #347964.
         #
-        if isinstance(orca_state.lastInputEvent,
-                      input_event.KeyboardEvent):
+        if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent) \
+           and orca_state.lastNonModifierKeyEvent:
             string = orca_state.lastNonModifierKeyEvent.event_string
             if string == "Delete":
                 rolesList = [pyatspi.ROLE_TABLE_CELL, \
@@ -1473,6 +1515,74 @@ class Script(default.Script):
         #
         default.Script.onFocus(self, event)
 
+    def onActiveDescendantChanged(self, event):
+        """Called when an object who manages its own descendants detects a
+        change in one of its children.
+
+        Arguments:
+        - event: the Event
+        """
+
+        # The default script's onActiveDescendantChanged method is cutting
+        # off speech with a speech.stop. If we're in the spellcheck dialog,
+        # this interrupts the presentation of the context.
+        #
+        if self.isSpellingSuggestionsList(event.source):
+            orca.setLocusOfFocus(event, event.any_data)
+
+            # We'll tuck away the activeDescendant information for future
+            # reference since the AT-SPI gives us little help in finding
+            # this.
+            #
+            if orca_state.locusOfFocus \
+               and (orca_state.locusOfFocus != event.source):
+                self.pointOfReference['activeDescendantInfo'] = \
+                    [orca_state.locusOfFocus.parent,
+                     orca_state.locusOfFocus.getIndexInParent()]
+            return
+
+        default.Script.onActiveDescendantChanged(self, event)
+
+    def onTextInserted(self, event):
+        """Called whenever text is inserted into an object.
+
+        Arguments:
+        - event: the Event
+        """
+
+        # When the active descendant in the list of misspelled words
+        # changes, we typically get an object:active-descendant-changed
+        # event. Unfortunately, we don't seem to get this event (or a
+        # focus: event) when the user presses a button without moving
+        # focus there explicitly. (e.g. pressing Alt+R) The label which
+        # is associated with the spelling list gets new text. So we'll
+        # try to look for that instead.
+        #
+        if event.source.getRole() == pyatspi.ROLE_LABEL:
+            relations = event.source.getRelationSet()
+            for relation in relations:
+                if relation.getRelationType() == pyatspi.RELATION_LABEL_FOR:
+                    target = relation.getTarget(0)
+                    if self.isSpellingSuggestionsList(target):
+                        [badWord, allTokens] = \
+                            self.getMisspelledWordAndBody(target,
+                                                          self.message_panel)
+                        self.speakMisspeltWord(allTokens, badWord)
+
+                        try:
+                            selection = target.querySelection()
+                        except NotImplementedError:
+                            selection = None
+                        if selection and selection.nSelectedChildren > 0:
+                            newFocus = selection.getSelectedChild(0)
+                            orca.setLocusOfFocus(event, newFocus)
+                            self.pointOfReference['activeDescendantInfo'] = \
+                                [target, newFocus.getIndexInParent()]
+
+                        return
+
+        default.Script.onTextInserted(self, event)
+
 # Values used to construct a time string for calendar appointments.
 #
 timeIncrements = {}
