@@ -21,9 +21,9 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 4471 2009-01-28 17:52:43Z wwalker $"
-__version__   = "$Revision: 4471 $"
-__date__      = "$Date: 2009-01-28 12:52:43 -0500 (Wed, 28 Jan 2009) $"
+__id__        = "$Id: settings.py 4507 2009-02-08 19:37:37Z wwalker $"
+__version__   = "$Revision: 4507 $"
+__date__      = "$Date: 2009-02-08 14:37:37 -0500 (Sun, 08 Feb 2009) $"
 __copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -720,6 +720,30 @@ textAttributesBrailleIndicator = TEXT_ATTR_BRAILLE_NONE
 #
 repeatCharacterLimit = 4
 
+# Tags associated with ARIA landmarks.
+#
+ariaLandmarks = [
+    "application",
+    "article",
+    "banner",
+    "complementary",
+    "contentinfo",
+    "definition",
+    "directory",
+    "document",
+    "grid",
+    "log",
+    "main",
+    "menubar",
+    "navigation",
+    "note",
+    "region",
+    "search",
+    "secondary",
+    "seealso",
+    "status",
+]
+
 # Script developer feature.  If False, just the default script
 # will be used.  Helps determine difference between custom
 # scripts and the default script behavior.
