@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 4417 2009-01-09 16:33:34Z wwalker $"
-__version__   = "$Revision: 4417 $"
-__date__      = "$Date: 2009-01-09 11:33:34 -0500 (Fri, 09 Jan 2009) $"
+__id__        = "$Id: speechgenerator.py 4471 2009-01-28 17:52:43Z wwalker $"
+__version__   = "$Revision: 4471 $"
+__date__      = "$Date: 2009-01-28 12:52:43 -0500 (Wed, 28 Jan 2009) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -37,7 +37,7 @@ import settings
 
 from orca_i18n import _         # for gettext support
 from orca_i18n import ngettext  # for ngettext support
-from orca_i18n import Q_        # to provide qualified translatable strings
+from orca_i18n import C_        # to provide qualified translatable strings
 
 class SpeechGenerator:
     """Takes accessible objects and produces a string to speak for
@@ -155,25 +155,53 @@ class SpeechGenerator:
         self.speechGenerators[pyatspi.ROLE_WINDOW]              = \
              self._getSpeechForWindow
 
-    def _getSpeechForObjectAccelerator(self, obj):
-        """Returns a list of utterances that describes the keyboard
-        accelerator (and possibly shortcut) for the given object.
+    def _addSpeechForObjectAccelerator(self, obj, utterances):
+        """Adds an utterance that describes the keyboard accelerator for the
+        given object to the list of utterances passed in.
 
         Arguments:
         - obj: the Accessible object
+        - utterances: the list of utterances to add to.
 
         Returns a list of utterances to be spoken.
         """
 
-        utterances = []
+        if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
+            [mnemonic, shortcut, accelerator] = self._script.getKeyBinding(obj)
+            if accelerator:
+                # Add punctuation for better prosody.
+                #
+                #if utterances:
+                #    utterances[-1] += "."
+                utterances.append(accelerator)
 
-        result = self._script.getKeyBinding(obj)
-        accelerator = result[2]
+    def _addSpeechForObjectMnemonic(self, obj, utterances):
+        """Adds an utterance that describes the mnemonic for the given object
+        to the utterances passed in.
 
-        if len(accelerator) > 0:
-            utterances.append(accelerator)
+        Arguments:
+        - obj: the Accessible object
+        - utterances: the list of utterances to add to.
+        """
 
-        return utterances
+        if settings.enableMnemonicSpeaking:
+            # The mnemonic is what we're really looking for.  But,
+            # some objects (e.g., menu items) only expose the mnemonic
+            # via the full shortcut (e.g., "Alt f c" for a "Close"
+            # menu item).  So, we fall back to the last character in
+            # the shortcut if the shortcut exists.
+            #
+            [mnemonic, shortcut, accelerator] = self._script.getKeyBinding(obj)
+            if mnemonic:
+                mnemonic = mnemonic[-1] # we just want a single character
+            if not mnemonic and shortcut:
+                mnemonic = shortcut
+            if mnemonic:
+                # Add punctuation for better prosody.
+                #
+                #if utterances:
+                #    utterances[-1] += "."
+                utterances.append(mnemonic)
 
     def _getSpeechForObjectAvailability(self, obj):
         """Returns a list of utterances that describes the availability
@@ -237,9 +265,7 @@ class SpeechGenerator:
                     # Translators: when the user selects (highlights) text in
                     # a document, Orca lets them know this.
                     #
-                    # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                    #
-                    utterance = [Q_("text|selected")]
+                    utterance = [C_("text", "selected")]
 
         return utterance
 
@@ -294,7 +320,7 @@ class SpeechGenerator:
 
         The default speech will be of the following form:
 
-        label name role availability
+        label name role availability mnemonic
 
         Arguments:
         - obj: an Accessible
@@ -315,8 +341,9 @@ class SpeechGenerator:
                 utterances.extend(name)
             utterances.extend(self._getSpeechForAllTextSelection(obj))
             utterances.extend(self.getSpeechForObjectRole(obj, role))
-
-        utterances.extend(self._getSpeechForObjectAvailability(obj))
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+            if obj == orca_state.locusOfFocus:
+                self._addSpeechForObjectMnemonic(obj, utterances)
 
         self._debugGenerator("_getDefaultSpeech",
                              obj,
@@ -416,6 +443,7 @@ class SpeechGenerator:
         """
 
         utterances = []
+
         state = obj.getState()
         if state.contains(pyatspi.STATE_INDETERMINATE):
             # Translators: this represents the state of a checkbox.
@@ -445,8 +473,9 @@ class SpeechGenerator:
             else:
                 utterances.extend(self.getSpeechForObjectRole(obj))
             utterances.append(checkedState)
-            utterances.extend(self._getSpeechForObjectAvailability(obj))
             utterances.extend(self._getSpeechForRequiredObject(obj))
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
         else:
             utterances.append(checkedState)
 
@@ -470,9 +499,8 @@ class SpeechGenerator:
 
         utterances = self._getSpeechForCheckBox(obj, already_focused)
 
-        if (settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE)\
-           and not already_focused:
-            utterances.extend(self._getSpeechForObjectAccelerator(obj))
+        if not already_focused:
+            self._addSpeechForObjectAccelerator(obj, utterances)
 
         self._debugGenerator("_getSpeechForCheckMenuItem",
                              obj,
@@ -526,12 +554,13 @@ class SpeechGenerator:
         if not already_focused:
             utterances.extend(self.getSpeechForObjectRole(obj))
 
-        utterances.extend(self._getSpeechForObjectAvailability(obj))
-
         for child in obj:
             if child.getRole() == pyatspi.ROLE_TEXT:
                 utterances.extend(self._getSpeechForAllTextSelection(child))
 
+        utterances.extend(self._getSpeechForObjectAvailability(obj))
+        self._addSpeechForObjectMnemonic(obj, utterances)
+
         self._debugGenerator("_getSpeechForComboBox",
                              obj,
                              already_focused,
@@ -687,7 +716,7 @@ class SpeechGenerator:
                                 alertAndDialogCount) % alertAndDialogCount
                 utterances.append(line)
 
-        utterances.extend(self._getSpeechForObjectAvailability(obj))
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
 
         self._debugGenerator("_getSpeechForFrame",
                              obj,
@@ -864,7 +893,7 @@ class SpeechGenerator:
         """
 
         utterances = []
- 
+
         if not already_focused:
             label = self._getSpeechForObjectLabel(obj)
             utterances.extend(label)
@@ -872,7 +901,6 @@ class SpeechGenerator:
             if name != label:
                 utterances.extend(name)
             utterances.extend(self._getSpeechForAllTextSelection(obj))
-        utterances.extend(self._getSpeechForObjectAvailability(obj))
 
         # If already in focus then the tree probably collapsed or expanded
         state = obj.getState()
@@ -890,6 +918,8 @@ class SpeechGenerator:
                 #
                 utterances.append(_("collapsed"))
                 
+        utterances.extend(self._getSpeechForObjectAvailability(obj))
+
         self._debugGenerator("_getSpeechForListItem",
                              obj,
                              already_focused,
@@ -906,12 +936,19 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
-        utterances = self._getDefaultSpeech(obj, already_focused)
+        utterances = []
 
-        if (obj == orca_state.locusOfFocus) \
-               and (settings.speechVerbosityLevel \
-                    == settings.VERBOSITY_LEVEL_VERBOSE):
-            utterances.extend(self._getSpeechForObjectAccelerator(obj))
+        if not already_focused:
+            label = self._getSpeechForObjectLabel(obj)
+            utterances.extend(label)
+            name = self._getSpeechForObjectName(obj)
+            if name != label:
+                utterances.extend(name)
+            utterances.extend(self._getSpeechForAllTextSelection(obj))
+            utterances.extend(self.getSpeechForObjectRole(obj))
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
+            self._addSpeechForObjectAccelerator(obj, utterances)
 
         self._debugGenerator("_getSpeechForMenu",
                              obj,
@@ -931,7 +968,6 @@ class SpeechGenerator:
         """
 
         utterances = self._getDefaultSpeech(obj, already_focused)
-
         self._debugGenerator("_getSpeechForMenuBar",
                              obj,
                              already_focused,
@@ -949,12 +985,15 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
+        utterances = []
+
         # No need to say "menu item" because we already know that.
         #
-        utterances = self._getSpeechForObjectName(obj)
-        if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
-            utterances.extend(self._getSpeechForObjectAvailability(obj))
-            utterances.extend(self._getSpeechForObjectAccelerator(obj))
+        label = self._getSpeechForObjectLabel(obj)
+        utterances.extend(label)
+        name = self._getSpeechForObjectName(obj)
+        if name != label:
+            utterances.extend(name)
 
         # OpenOffice check menu items currently have a role of "menu item"
         # rather then "check menu item", so we need to test if one of the
@@ -969,6 +1008,11 @@ class SpeechGenerator:
             #
             utterances.append(_("checked"))
 
+        if not already_focused:
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
+            self._addSpeechForObjectAccelerator(obj, utterances)
+
         self._debugGenerator("_getSpeechForMenuItem",
                              obj,
                              already_focused,
@@ -1008,6 +1052,8 @@ class SpeechGenerator:
 
         utterances.extend(self._getSpeechForAllTextSelection(obj))
 
+        self._addSpeechForObjectMnemonic(obj, utterances)
+
         self._debugGenerator("_getSpeechForText",
                              obj,
                              already_focused,
@@ -1146,21 +1192,18 @@ class SpeechGenerator:
         """
 
         utterances = []
+
         state = obj.getState()
         if state.contains(pyatspi.STATE_CHECKED):
             # Translators: this is in reference to a radio button being
             # selected or not.
             #
-            # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-            #
-            selectionState = Q_("radiobutton|selected")
+            selectionState = C_("radiobutton", "selected")
         else:
             # Translators: this is in reference to a radio button being
             # selected or not.
             #
-            # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-            #
-            selectionState = Q_("radiobutton|not selected")
+            selectionState = C_("radiobutton", "not selected")
 
         # If it's not already focused, say it's name
         #
@@ -1172,6 +1215,7 @@ class SpeechGenerator:
             utterances.append(selectionState)
             utterances.extend(self.getSpeechForObjectRole(obj))
             utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
         else:
             utterances.append(selectionState)
 
@@ -1187,17 +1231,17 @@ class SpeechGenerator:
         already had focus, then only the state is spoken.
 
         Arguments:
-        - obj: the check menu item
+        - obj: the radio menu item
         - already_focused: False if object just received focus
 
         Returns a list of utterances to be spoken for the object.
         """
 
-        utterances = self._getSpeechForRadioButton(obj, False)
+        utterances = []
+        utterances.extend(self._getSpeechForRadioButton(obj, False))
 
-        if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
-            utterances.extend(self._getSpeechForObjectAvailability(obj))
-            utterances.extend(self._getSpeechForObjectAccelerator(obj))
+        if not already_focused:
+            self._addSpeechForObjectAccelerator(obj, utterances)
 
         self._debugGenerator("_getSpeechForRadioMenuItem",
                              obj,
@@ -1280,9 +1324,9 @@ class SpeechGenerator:
                 utterances.extend(self._getSpeechForObjectName(obj))
             utterances.extend(self.getSpeechForObjectRole(obj))
             utterances.append(valueString)
-            utterances.extend(self._getSpeechForObjectAvailability(obj))
             utterances.extend(self._getSpeechForRequiredObject(obj))
-
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
         self._debugGenerator("_getSpeechForSlider",
                              obj,
                              already_focused,
@@ -1336,6 +1380,7 @@ class SpeechGenerator:
             utterances.extend(self.getSpeechForObjectRole(obj))
             utterances.append(valueString)
             utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
 
         self._debugGenerator("_getSpeechForSplitPane",
                              obj,
@@ -1701,6 +1746,7 @@ class SpeechGenerator:
         """
 
         utterances = []
+
         state = obj.getState()
         if state.contains(pyatspi.STATE_CHECKED) \
            or state.contains(pyatspi.STATE_PRESSED):
@@ -1723,6 +1769,7 @@ class SpeechGenerator:
             utterances.extend(self.getSpeechForObjectRole(obj))
             utterances.append(checkedState)
             utterances.extend(self._getSpeechForObjectAvailability(obj))
+            self._addSpeechForObjectMnemonic(obj, utterances)
         else:
             utterances.append(checkedState)
 
